import 'dart:convert';

import 'package:ambito/main.dart';
import 'package:ambito/src/entity/entities.dart';
import 'package:ambito/src/entity/measure/details/measure_details.dart';
import 'package:ambito/src/entity/measure/details/measure_details_repository.dart';
import 'package:ambito/src/entity/measure/general/measure_general_repository.dart';
import 'package:ambito/src/entity/measure/types_details/measure_types_details.dart';
import 'package:ambito/src/entity/measure/types_details/measure_types_details_repository.dart';
import 'package:ambito/src/packages/ambito_api/restclient.dart';
import 'package:get/get.dart';

import '../../config/config.dart';
import '../../entity/funding_program/funding_program_repository.dart';
import '../../entity/measure/measure_repository.dart';

class BaseApi {
  Map tables = {};

  init() {
    // read table ids from .env file...
    tables = baserowIds;
  }

  Future<bool> getContent(String table) async {
    init();
    int tableId = tables[table] ?? 0;

    if (tableId <= 0) return false;

    var response = await RestClient().get('$tableId/?size=200');
    if (response.statusCode != 200) Get.toNamed('/error');

    var json = _jsonDecoded(response.body);

    var results = json['results'];

    final repositoryMap = {
      //'tree_type': TreeTypeRepository().put,
      //'measure': MeasureRepository().putMeasureGroups,
      'measure_types': MeasureRepository().put,
      'measure_general': MeasureGeneralRepository().put,
      'measure_details': MeasureDetailsRepository().put,
      'measure_types_details': MeasureTypesDetailsRepository().put,
      //'measure_combination': MeasureCombinationRepository().put,
      //'organism': OrganismRepository().put,
      'funding_program': FundingProgramRepository().put,
      //'location_requirements': LocationRequirementsRepository().put,
      //'reference_implementation': ReferenceImplementationRepository().put,
      //'business': BusinessRepository().put,
      //'service_provider': ServiceProviderRepository().put,
      //'service_provider_contact_person':
      //    ServiceProviderContactPersonRepository().put,
      //'material': MaterialRepository().put,
      //'source': SourceRepository().put,
      //'experience_report': ExperienceReportRepository().put,
      //'approval_requirement': ApprovalRequirementRepository().put
    };

    var repositoryFunction = repositoryMap[table];
    if (repositoryFunction == null) return false;

    for (var item in results) {
      var model = _createModelFromJson(table, item);
      repositoryFunction(model);
    }

    return true;
  }

  dynamic _createModelFromJson(String table, Map<String, dynamic> json) {
    switch (table) {
      case 'measure_types':
        final measureType = MeasureTypes.fromJson(json);
        listMeasures.add(measureType.toListMeasure());
        return measureType;
      case 'measure_general':
        final measureGeneral = MeasureGeneral.fromJson(json);
        listMeasures.add(measureGeneral.toListMeasure());
        return measureGeneral;
      case 'measure_details':
        return MeasureDetails.fromJson(json);
      case 'measure_types_details':
        return MeasureTypesDetails.fromJson(json);
      case 'funding_program':
        return FundingProgram.fromJson(json);
      default:
        return null;
    }
  }

  dynamic _jsonDecoded(String input) {
    return jsonDecode(utf8.decode(input.runes.toList()));
  }
}
