import 'package:ambito/src/consts/consts.dart';
import 'package:ambito/src/extensions/extensions.dart';
import 'package:flutter/material.dart';

import '../../../../entity/entities.dart';
import '../../../../packages/ambito_theme/ambito_theme.dart';
import '_main_card.dart';

class CreationCard extends StatelessWidget {
  const CreationCard({super.key, required this.massnahme});

  final MeasureTypes massnahme;

  @override
  Widget build(BuildContext context) {
    final AmbitoTheme theme = getTheme(context);

    Chip buildChip(String label, [active = false]) {
      return Chip(
        labelPadding: EdgeInsets.zero,
        backgroundColor: active
            ? theme.currentColorScheme.secondary
            : theme.currentColorScheme.outline.withOpacity(.1),
        label: Text(
          label,
          style: theme.currentThemeData.textTheme.bodyLarge?.copyWith(
            color: active
                ? theme.currentColorScheme.onSecondary
                : theme.currentColorScheme.onSurface,
          ),
        ),
      );
    }

    List<Chip> buildChips(List<String?>? items) =>
        months.map((item) => buildChip(item, items?.contains(item))).toList();

    List<Widget> buildTools(List<String?>? tools) =>
        tools?.where((item) => item?.isNotEmpty ?? false).map((item) {
          return Row(
            children: [
              const Icon(Icons.fiber_manual_record, size: 8),
              theme.horizontalSpacerSmall,
              Expanded(
                child: Text(
                  item!,
                  style: theme.currentThemeData.textTheme.bodyLarge,
                ),
              ),
            ],
          );
        }).toList() ??
        [];

    Widget buildSection({
      required String titleKey,
      required List<Widget> children,
    }) {
      if (children.isEmpty) return const SizedBox.shrink();
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          const Divider(),
          Text(
            context.translate(titleKey),
            style: theme.currentThemeData.textTheme.labelMedium,
          ),
          ...children,
        ],
      );
    }

    Widget buildTextSection(String? text, String titleKey) {
      if (text == null || text.isEmpty) return const SizedBox.shrink();
      return buildSection(
        titleKey: titleKey,
        children: [
          Text(
            text,
            style: theme.currentThemeData.textTheme.bodyLarge,
          ),
        ],
      );
    }

    final tfChips =
        buildChips(massnahme.applyTimeFrame?.map((e) => e.value).toList());
    final creationTools =
        buildTools(massnahme.applyTools?.map((e) => e.value).toList());

    return MainCard(
      massnahme: massnahme,
      title: 'page.actionDetailPage.creation.title',
      content: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          if (tfChips.isNotEmpty)
            buildSection(
              titleKey: 'page.actionDetailPage.creation.timeFrame',
              children: [
                Wrap(
                  runSpacing: 8,
                  spacing: 8,
                  children: tfChips,
                ),
              ],
            ),
          buildTextSection(
            massnahme.applyTimeFrameAdditional,
            'page.actionDetailPage.creation.additional',
          ),
          buildTextSection(
            massnahme.applyReminder,
            'page.actionDetailPage.creation.reminders',
          ),
          if (creationTools.isNotEmpty)
            buildSection(
              titleKey: 'page.actionDetailPage.creation.tools',
              children: creationTools,
            ),
          buildTextSection(
            massnahme.applyWorkSteps,
            'page.actionDetailPage.creation.steps',
          ),
          buildTextSection(
            massnahme.applyTips,
            'page.actionDetailPage.creation.tips',
          ),
        ],
      ),
    );
  }
}
