import 'package:ambito/src/extensions/extensions.dart';
import 'package:flutter/material.dart';

import '../../../../entity/entities.dart';
import '../../../../packages/ambito_theme/ambito_theme.dart';
import '_main_card.dart';

class MaintenanceCard extends StatelessWidget {
  const MaintenanceCard({super.key, required this.massnahme});

  final MeasureTypes massnahme;

  @override
  Widget build(BuildContext context) {
    final AmbitoTheme theme = getTheme(context);

    Chip buildChip(String label) {
      return Chip(
        labelPadding: EdgeInsets.zero,
        backgroundColor: theme.currentColorScheme.secondary,
        label: Text(
          label,
          style: theme.currentThemeData.textTheme.bodyLarge
              ?.copyWith(color: theme.currentColorScheme.onSecondary),
        ),
      );
    }

    List<Chip> buildChips(List<String?>? items) =>
        items
            ?.where((item) => item?.isNotEmpty ?? false)
            .map((item) => buildChip(item!))
            .toList() ??
        [];

    List<Widget> buildTools(List<String?>? tools) =>
        tools?.where((item) => item?.isNotEmpty ?? false).map((item) {
          return Row(
            children: [
              const Icon(Icons.fiber_manual_record, size: 8),
              theme.horizontalSpacerSmall,
              Expanded(
                child: Text(
                  item!,
                  style: theme.currentThemeData.textTheme.bodyLarge,
                ),
              ),
            ],
          );
        }).toList() ??
        [];

    Widget buildSection({
      required String titleKey,
      required List<Widget> children,
    }) {
      if (children.isEmpty) return const SizedBox.shrink();
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          const Divider(),
          Text(
            context.translate(titleKey),
            style: theme.currentThemeData.textTheme.labelMedium,
          ),
          ...children,
        ],
      );
    }

    Widget buildTextSection(String? text, String titleKey) {
      if (text == null || text.isEmpty) return const SizedBox.shrink();
      return buildSection(
        titleKey: titleKey,
        children: [
          Text(
            text,
            style: theme.currentThemeData.textTheme.bodyLarge,
          ),
        ],
      );
    }

    final tfChips = buildChips(
        massnahme.maintenanceTimeFrame?.map((e) => e.value).toList());
    final tfMaintenance = buildChips(
        massnahme.maintenanceFrequency?.map((e) => e.value).toList());
    final creationTools =
        buildTools(massnahme.maintenanceMaterial?.map((e) => e.value).toList());

    return MainCard(
      massnahme: massnahme,
      title: 'page.actionDetailPage.maintenance.title',
      content: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          if (tfChips.isNotEmpty)
            buildSection(
              titleKey: 'page.actionDetailPage.maintenance.time',
              children: [
                Wrap(
                  runSpacing: 8,
                  spacing: 8,
                  children: tfChips,
                ),
              ],
            ),
          buildTextSection(
            massnahme.maintenanceTimeFrameAdditional,
            'page.actionDetailPage.maintenance.additional_time',
          ),
          if (tfMaintenance.isNotEmpty)
            buildSection(
              titleKey: 'page.actionDetailPage.maintenance.frequency',
              children: [
                Wrap(
                  runSpacing: 8,
                  spacing: 8,
                  children: tfMaintenance,
                ),
              ],
            ),
          buildTextSection(
            massnahme.maintenanceFrequencyAdditional,
            'page.actionDetailPage.maintenance.additional_frequency',
          ),
          if (creationTools.isNotEmpty)
            buildSection(
              titleKey: 'page.actionDetailPage.maintenance.tools',
              children: creationTools,
            ),
          buildTextSection(
            massnahme.maintenanceWorkSteps,
            'page.actionDetailPage.maintenance.steps',
          ),
          buildTextSection(
            massnahme.maintenanceTips,
            'page.actionDetailPage.maintenance.tips',
          ),
        ],
      ),
    );
  }
}
