import 'package:ambito/src/extensions/extensions.dart';
import 'package:ambito/src/widgets/buttons/text_button.dart';
import 'package:flutter/material.dart';

import '../../../../entity/entities.dart';
import '../../../../packages/ambito_theme/ambito_theme.dart';
import '_sidebar_card.dart';

class MaterialCard extends StatelessWidget {
  const MaterialCard({super.key, required this.massnahme});

  final MeasureTypes massnahme;

  @override
  Widget build(BuildContext context) {
    final AmbitoTheme theme = getTheme(context);

    return SideBarCard(
      massnahme: massnahme,
      title: 'page.actionDetailPage.material.title',
      content: [
        _getSortedMaterials(context),
        theme.verticalSpacer,
        if (massnahme.applyMaterialCosts != null)
          Text(context
                  .translate('page.actionDetailPage.material.suggested_price') +
              massnahme.applyMaterialCosts +
              ' €'),
        theme.verticalSpacerSmall,
        Align(
          alignment: Alignment.center,
          child: TextButtonElement(
            title: 'page.actionDetailPage.material.add_to_basket',
            onPressed: () {},
          ),
        )
      ],
    );
  }

  _getSortedMaterials(BuildContext context) {
    final AmbitoTheme theme = getTheme(context);

    List<String> items = [];
    if (massnahme.applyMaterial != null) {
      for (IdValueColor ivc in massnahme.applyMaterial!) {
        items.add(ivc.value!);
      }
    }
    items.sort((a, b) => a.toString().compareTo(b.toString()));
    List<Widget> tiles = [];
    for (String item in items) {
      tiles.add(
        Row(
          children: [
            const Icon(
              Icons.fiber_manual_record,
              size: 8,
            ),
            theme.horizontalSpacerSmall,
            Expanded(
              child: Text(
                item,
                style: theme.currentThemeData.textTheme.bodyLarge,
              ),
            )
          ],
        ),
      );
    }
    return Column(
      children: tiles,
    );
  }
}
