import 'package:ambito/src/entity/entities.dart';
import 'package:ambito/src/entity/measure/measure_repository.dart';
import 'package:ambito/src/extensions/extensions.dart';
import 'package:animated_segmented_tab_control/animated_segmented_tab_control.dart';
import 'package:expansion_tile_card/expansion_tile_card.dart';
import 'package:flutter/material.dart';
import 'package:flutter_breadcrumb/flutter_breadcrumb.dart';
import 'package:get/get.dart';
import 'package:screen_breakpoints/screen_breakpoints.dart';

import '../../../config/config.dart';
import '../../../packages/ambito_theme/ambito_theme.dart';
import '../../../widgets/appbar/ambito_appbar.dart';
import '../../ambito_page.dart';
import 'cards/_cards.dart';

final Map<String, GlobalKey<ExpansionTileCardState>> globalKeys = {};

class MeasureDetailPage extends AmbitoPage {
  const MeasureDetailPage({super.key});

  @override
  final String path = 'massnahme';
  @override
  final String title = 'Maßname';

  @override
  State<StatefulWidget> createState() => MeasureDetailPageState();
}

class MeasureDetailPageState extends State<MeasureDetailPage> {
  late final String id;
  final ScrollController scrollController = ScrollController();
  bool showBackToTopButton = false;
  final Map<String, GlobalKey<ExpansionTileCardState>> expansionKeys = {};
  MeasureTypes? massnahme;
  List<Widget> contentItems = [];
  final Set<String> visibleItems = {};
  bool collapsed = false;

  @override
  void initState() {
    super.initState();

    id = Get.parameters['id'] ?? '';
    if (id.isNotEmpty) {
      massnahme = MeasureRepository().get(int.parse(id)) as MeasureTypes;
    }

    scrollController.addListener(() {
      const showOffset = 10.0;
      setState(
          () => showBackToTopButton = scrollController.offset > showOffset);
    });
  }

  @override
  Widget build(BuildContext context) {
    final AmbitoTheme theme = getTheme(context);
    return Scaffold(
      appBar: AmbitoAppbar(
        links: const ['dashboard', 'massnahmen'],
        breakpoint: Breakpoint.fromContext(context),
        theme: theme,
      ),
      body: (context.breakpoint != myBreakpoints.sm)
          ? _buildInfoPage(context, massnahme)
          : _tabbedInfoPage(context, massnahme),
      floatingActionButton: (context.breakpoint != myBreakpoints.sm)
          ? AnimatedOpacity(
              duration: const Duration(milliseconds: 1000),
              opacity: showBackToTopButton ? 1.0 : 0.0,
              child: FloatingActionButton(
                shape: const CircleBorder(),
                elevation: 0,
                onPressed: () => scrollController.animateTo(
                  0,
                  duration: const Duration(milliseconds: 500),
                  curve: Curves.fastOutSlowIn,
                ),
                backgroundColor: theme.currentColorScheme.primary,
                child: Icon(Icons.arrow_upward,
                    color: theme.currentColorScheme.onPrimary),
              ),
            )
          : null,
    );
  }

  Widget _tabbedInfoPage(BuildContext context, MeasureTypes? massnahme) {
    final AmbitoTheme theme = getTheme(context);

    if (massnahme == null) {
      return const Center(child: CircularProgressIndicator());
    }

    return Padding(
      padding: context.breakpoint.padding,
      child: DefaultTabController(
        length: 2,
        child: Column(
          children: [
            Align(
              alignment: Alignment.centerLeft,
              child: Text(
                massnahme.measureType!,
                style: theme.currentThemeData.textTheme.headlineMedium,
              ),
            ),
            theme.verticalSpacerSmall,
            SegmentedTabControl(
              tabTextColor: theme.currentColorScheme.secondary,
              selectedTabTextColor: theme.currentColorScheme.onSecondary,
              barDecoration: BoxDecoration(
                color: theme.currentColorScheme.onSecondary,
                border: Border.all(
                  width: 1.0,
                  color: theme.currentColorScheme.secondary,
                ),
                borderRadius: const BorderRadius.all(
                  Radius.circular(
                    8.0,
                  ),
                ),
              ),
              indicatorDecoration:
                  BoxDecoration(color: theme.currentColorScheme.secondary),
              tabs: [
                SegmentTab(
                    label:
                        context.translate('page.actionDetailPage.tabs.info')),
                SegmentTab(
                    label:
                        context.translate('page.actionDetailPage.tabs.misc')),
              ],
            ),
            theme.verticalSpacerSmall,
            Expanded(
              child: TabBarView(
                children: [
                  SingleChildScrollView(
                    child: Column(children: _buildContentItems(massnahme)),
                  ),
                  SingleChildScrollView(
                    child: Column(
                      children: _buildSidebarItems(context, theme, massnahme),
                    ),
                  ),
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget _buildInfoPage(BuildContext context, MeasureTypes? massnahme) {
    final AmbitoTheme theme = getTheme(context);

    if (massnahme == null) {
      return const Center(child: CircularProgressIndicator());
    }

    contentItems = _buildContentItems(massnahme);

    return Column(
      children: [
        _buildBreadcrumb(context, theme, massnahme),
        SizedBox(
            width: double.infinity,
            height: 300,
            child: massnahme.getFullImage()),
        Expanded(
          child: SingleChildScrollView(
            controller: scrollController,
            child: SizedBox(
              width: 1152,
              child: Column(
                children: [
                  theme.verticalSpacer,
                  Align(
                    alignment: Alignment.centerLeft,
                    child: Text(
                      massnahme.measureType!,
                      style: theme.currentThemeData.textTheme.headlineMedium,
                    ),
                  ),
                  theme.verticalSpacer,
                  Row(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Expanded(
                        child: Column(children: contentItems),
                      ),
                      theme.horizontalSpacer,
                      SizedBox(
                        width: 300,
                        child: Column(
                          children:
                              _buildSidebarItems(context, theme, massnahme),
                        ),
                      ),
                    ],
                  ),
                ],
              ),
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildBreadcrumb(
      BuildContext context, AmbitoTheme theme, MeasureTypes massnahme) {
    return Align(
      alignment: Alignment.centerLeft,
      child: BreadCrumb(
        items: [
          BreadCrumbItem(
            content: TextButton(
              onPressed: () => Get.offAndToNamed('/'),
              child: Text('Massnahmenkategorien', style: theme.bodyMedium),
            ),
          ),
          BreadCrumbItem(
            content: TextButton(
              onPressed: () => Get.offAndToNamed('/massnahmendatenbank'),
              child: Text('Maßnamentypen', style: theme.bodyMedium),
            ),
          ),
          BreadCrumbItem(
            content: TextButton(
              onPressed: () => Get.offAndToNamed('/massnahmendatenbank'),
              child:
                  Text(massnahme.measureGroup!.value!, style: theme.bodyMedium),
            ),
          ),
          BreadCrumbItem(
              content:
                  Text(massnahme.measureType ?? '', style: theme.bodyMedium)),
        ],
        divider: const Icon(Icons.chevron_right),
      ),
    );
  }

  List<Widget> _buildContentItems(MeasureTypes massnahme) {
    final AmbitoTheme theme = getTheme(context);

    List<Widget> elements = [];

    void addCard(
        {required Widget card,
        required String titleKey,
        required bool condition}) {
      if (condition) {
        visibleItems.add(context.translate(titleKey));
        elements.addAll([card, theme.verticalSpacer]);
      }
    }

    addCard(
      card: DescriptionCard(massnahme: massnahme),
      titleKey: 'page.actionDetailPage.description.title',
      condition: massnahme.description?.isNotEmpty ?? false,
    );

    addCard(
      card: PresetsCard(massnahme: massnahme),
      titleKey: 'page.actionDetailPage.presets.title',
      condition: massnahme.objective?.isNotEmpty ?? false,
    );

    addCard(
      card: SizeCard(massnahme: massnahme),
      titleKey: 'page.actionDetailPage.size.title',
      condition: massnahme.applySize?.isNotEmpty ?? false,
    );

    addCard(
      card: LocationCard(massnahme: massnahme),
      titleKey: 'page.actionDetailPage.location.title',
      condition: massnahme.applyLocation?.isNotEmpty ?? false,
    );

    addCard(
      card: ImplementationCard(massnahme: massnahme),
      titleKey: 'page.actionDetailPage.implementation.title',
      condition: (massnahme.applyCosts?.isNotEmpty ?? false) &&
          (massnahme.applyTime?.isNotEmpty ?? false),
    );

    addCard(
      card: CreationCard(massnahme: massnahme),
      titleKey: 'page.actionDetailPage.creation.title',
      condition: massnahme.applyTimeFrame?.isNotEmpty ?? false,
    );

    addCard(
      card: MaintenanceCard(massnahme: massnahme),
      titleKey: 'page.actionDetailPage.maintenance.title',
      condition: massnahme.maintenanceTimeFrame?.isNotEmpty ?? false,
    );

    addCard(
      card: SourcesCard(massnahme: massnahme),
      titleKey: 'page.actionDetailPage.sources.title',
      condition: massnahme.sources?.isNotEmpty ?? false,
    );

    return [
      Align(
        alignment: Alignment.centerRight,
        child: TextButton(
          onPressed: () {
            globalKeys.forEach((key, value) {
              if (collapsed == false) {
                value.currentState?.collapse();
              } else {
                value.currentState?.expand();
              }
            });
            setState(() {
              collapsed = !collapsed;
            });
          },
          child: Text(
            context.translate((collapsed == false)
                ? 'page.actionDetailPage.expanded.collapse_all'
                : 'page.actionDetailPage.expanded.expand_all'),
            style: theme.bodyLarge.copyWith(
              color: theme.currentColorScheme.onSurface,
              decoration: TextDecoration.underline,
            ),
          ),
        ),
      ),
      theme.verticalSpacer,
      ...elements,
    ];
  }

  List<Widget> _buildSidebarItems(
      BuildContext context, AmbitoTheme theme, MeasureTypes massnahme) {
    return [
      if (visibleItems.isNotEmpty && context.breakpoint != myBreakpoints.sm)
        TocCard(massnahme: massnahme, items: visibleItems.toList()),
      if (visibleItems.isNotEmpty && context.breakpoint != myBreakpoints.sm)
        theme.verticalSpacer,
      AdvisorCard(massnahme: massnahme),
      theme.verticalSpacer,
      MaterialCard(massnahme: massnahme),
      theme.verticalSpacer,
      FactsheetCard(massnahme: massnahme),
      theme.verticalSpacer,
      ReviewCard(massnahme: massnahme),
    ];
  }
}
